@@ -1,16 +1,20 @@
 from __future__ import annotations
 
 import argparse
+import difflib
 import sys
 from pydoc import pager
 from typing import NoReturn
 from typing import Sequence
+from urllib.error import HTTPError
 from urllib.parse import urljoin
 from urllib.request import Request
 from urllib.request import urlopen
 
 
-__version__ = "1.0.0"
+__version__ = "1.1.0"
+
+REQUEST_TIMEOUT = 10  # seconds
 
 
 class Defaults:
@@ -22,15 +26,17 @@ def cli() -> NoReturn:
     raise SystemExit(main())
 
 
-def main() -> int:
+def main() -> int | str:
     parser = create_parser()
     args = parser.parse_args()
 
-    if hasattr(args, "handler"):
+    try:
         return args.handler(args)
-
-    parser.print_help()
-    return 1
+    except Exception as e:
+        if args.debug:
+            raise
+        else:
+            return str(e)
 
 
 def create_parser(
@@ -57,6 +63,13 @@ def create_parser(
         help="Write template list to stdout. By default, this program attempts "
         "to paginate the list of available templates for easier reading.",
     )
+    parser.add_argument(
+        "-d",
+        "--debug",
+        action="store_true",
+        default=False,
+        help="Increase program verbosity.",
+    )
 
     parser.set_defaults(handler=handler)
 
@@ -69,18 +82,11 @@ def handler(args: argparse.Namespace) -> int:
     no_pager: bool = args.no_pager
 
     if shell:
-        try:
-            completion_fn = globals()[f"{shell}_completion"]
-            print(completion_fn(), file=sys.stdout)
-        except KeyError:
-            print(_unknown_completion_shell_msg(shell))
-            return 1
+        text = generate_completion_str(shell)
+        print(text, file=sys.stdout)
     elif len(templates) == 0:
         text = "\n".join(list_templates())
-        if no_pager:
-            print(text, file=sys.stdout)
-        else:
-            pager(text)
+        print(text, file=sys.stdout) if no_pager else pager(text)
     else:
         text = create(templates)
         print(text, file=sys.stdout)
@@ -89,15 +95,35 @@ def handler(args: argparse.Namespace) -> int:
 
 
 def list_templates() -> list[str]:
-    res = _GET(_list_endpoint())
-    return res.decode("utf8").replace("\n", ",").split(",")
+    try:
+        res = _GET(_list_endpoint())
+    except Exception as e:
+        raise ApplicationError("Failed to fetch available templates") from e
+
+    return res.decode("utf8").replace(",", "\n").split()
 
 
 def create(templates: Sequence[str]) -> str:
-    res = _GET(_create_endpoint(templates))
+    try:
+        res = _GET(_create_endpoint(templates))
+    except HTTPError as e:
+        known_templates = list_templates()
+        raise TemplateNotFoundError(templates, known_templates) from e
+    except Exception as e:
+        raise ApplicationError("Failed to create .gitignore") from e
+
     return res.decode()
 
 
+def generate_completion_str(shell: str) -> str:
+    try:
+        completion_fn = globals()[f"{shell}_completion"]
+        return completion_fn()
+    except KeyError:
+        msg = _unknown_completion_shell_msg(shell)
+        raise ApplicationError(msg)
+
+
 def bash_completion() -> str:
     content = _BASH_COMPLETION_TEMPLATE.format(all_templates=" ".join(list_templates()))
     return content
@@ -122,17 +148,62 @@ def _create_endpoint(templates: Sequence[str], *, base: str | None = None) -> st
     return urljoin(base, url)
 
 
-def _GET(url: str) -> bytes:
+def _GET(url: str, timeout: float = 10.0) -> bytes:
     headers = {"User-Agent": "Mozilla/5.0"}
     request = Request(url, headers=headers)
-    with urlopen(request) as response:
+    with urlopen(request, timeout=timeout) as response:
         return response.read()
 
 
+class ApplicationError(Exception):
+    """Generic exception type for this package to use"""
+
+
+class TemplateNotFoundError(ApplicationError):
+    def __init__(
+        self,
+        templates: Sequence[str],
+        known_templates: Sequence[str] | None = None,
+    ):
+        self.templates = templates
+        self.known_templates = known_templates or {}
+
+        unknown_templates = set(self.templates) - set(self.known_templates)
+        if unknown_templates:
+            self.msg = _unknown_templates_msg(templates, known_templates)
+        else:
+            self.msg = "Failed to create .gitignore"
+
+        super().__init__(self.msg)
+
+
 def _unknown_completion_shell_msg(shell: str):
     return f"Unknown shell {shell!r}. Expected one of {Defaults.completion_shells!r}"
 
 
+def _unknown_templates_msg(
+    templates: Sequence[str],
+    known_templates: Sequence[str] | None = None,
+) -> str:
+    lines = ["Encountered unknown templates:"]
+    if known_templates:
+        for template in templates:
+            if template in known_templates:
+                continue
+            line = f"- No available template with the name {template!r}."
+            matches = difflib.get_close_matches(template, known_templates, 5)
+            if len(matches) > 0:
+                line += f" Did you mean {_oxfordcomma(matches, 'or')}?"
+            lines.append(line)
+    return "\n".join(lines)
+
+
+def _oxfordcomma(entries: Sequence[str], conjunction: str = "and"):
+    if len(entries) <= 2:
+        return f" {conjunction} ".join(entries)
+    return f"{', '.join(entries[:-1])}, {conjunction} {entries[-1]}"
+
+
 _BASH_COMPLETION_TEMPLATE = """\
 #!/usr/bin/env bash
 complete -W "{all_templates}" gi
@@ -147,5 +218,6 @@ complete -c gi -l no-pager -d 'Do not pipe output into a pager'
 complete -c gi -l completion -a '{all_shells}' -d 'Generate shell completion file'
 """
 
+
 if __name__ == "__main__":
     cli()
